@@ -1,8 +1,8 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2005-2007 Nicolas Deram <nicolas@itaapy.com>
+# Copyright (C) 2005-2008 Nicolas Deram <nicolas@itaapy.com>
 # Copyright (C) 2007 Henry Obein <henry@itaapy.com>
-# Copyright (C) 2007 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2007 Sylvain Taverne <sylvain@itaapy.com>
+# Copyright (C) 2007-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
+# Copyright (C) 2007-2008 Sylvain Taverne <sylvain@itaapy.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,11 +21,11 @@
 from datetime import datetime
 
 # Import from itools
-from itools.catalog import (get_field, PhraseQuery, EqQuery, AndQuery,
-    MemoryCatalog)
-from itools.datatypes import DateTime, String, Integer
+from itools.datatypes import DateTime, String, Integer, Unicode, is_datatype
 from itools.handlers import File
 from itools import vfs
+from itools.xapian import AndQuery, PhraseQuery, get_field
+from memory import MemoryCatalog
 from parser import parse
 
 
@@ -265,6 +265,31 @@ def parse_table(data):
 
 
 
+###########################################################################
+# Helper functions
+###########################################################################
+def is_multilingual(datatype):
+    return is_datatype(datatype, Unicode) and datatype.multiple
+
+
+
+###########################################################################
+# UniqueError
+###########################################################################
+class UniqueError(ValueError):
+    """Raised when setting a value already used to a unique property.
+    """
+
+    def __init__(self, name, value):
+        self.name = name
+        self.value = value
+
+
+    def __str__(self):
+        return (u'Error: Field "$field" must be unique, '
+                u'value "$value" is already used.')
+
+
 ###########################################################################
 # File Handler
 ###########################################################################
@@ -278,9 +303,9 @@ class Property(object):
 
     __slots__ = ['value', 'parameters']
 
-    def __init__(self, value, parameters={}):
+    def __init__(self, value, **kw):
         self.value = value
-        self.parameters = parameters
+        self.parameters = kw
 
 
 
@@ -302,7 +327,7 @@ class Record(list):
                 self.__class__.__name__, name)
 
         property = version[name]
-        if isinstance(property, list):
+        if type(property) is list:
             return [ x.value for x in property ]
         return property.value
 
@@ -322,7 +347,7 @@ class Record(list):
             return None
 
         property = version[name]
-        if isinstance(property, list):
+        if type(property) is list:
             return [ x.value for x in property ]
         return property.value
 
@@ -331,19 +356,25 @@ class Record(list):
 class Table(File):
 
     record_class = Record
+    incremental_save = True
 
     #######################################################################
     # Hash with field names and its types
     # Example: {'firstname': Unicode, 'lastname': Unicode, 'age': Integer}
     # To index some fields the schema should be declared as:
-    # schema = {'firstname': Unicode, 'lastname': Unicode,
-    #           'age': Integer(index='<analyser>')}
-    # where <analyser> is an itools.catalog analyser or derivate: keyword,
+    # record_schema = {'firstname': Unicode, 'lastname': Unicode,
+    #                  'age': Integer(index='<analyser>')}
+    # where <analyser> is an itools.xapian analyser or derivate: keyword,
     # book, text, path.
     #######################################################################
     schema = {}
+    record_schema = {}
+    parameters_schema = {
+        'language': String(multiple=False)}
+
 
     def get_datatype(self, name):
+        # Table schema
         if name == 'ts':
             return DateTime(multiple=False)
         if name in self.schema:
@@ -351,55 +382,91 @@ class Table(File):
         return String(multiple=True)
 
 
-    def properties_to_dict(self, properties, version=None):
+    def get_record_datatype(self, name):
+        # Record schema
+        if name == 'ts':
+            return DateTime(multiple=False)
+        if name in self.record_schema:
+            return self.record_schema[name]
+        # FIXME Probably we should raise an exception here
+        return String(multiple=True)
+
+
+    def get_parameter_datatype(self, name):
+        # Record schema
+        schema = self.parameters_schema
+        if name in schema:
+            return schema[name]
+        return String
+
+
+    def properties_to_dict(self, properties, version=None, first=False):
         """Add the given "properties" as Property objects or Property objects
         list to the given dictionnary "version".
         """
         if version is None:
             version = {}
-        # Fix the type
-        for name, value in properties.items():
-            datatype = self.get_datatype(name)
-            if getattr(datatype, 'multiple', False) is True:
-                if isinstance(value, list):
-                    version[name] = []
-                    for x in value:
-                        if isinstance(x, Property):
-                            version[name].append(x)
-                        else:
-                            version[name].append(Property(x))
-                elif isinstance(value, Property):
-                    version[name] = [value]
-                else:
-                    version[name] = [Property(value)]
-            elif not isinstance(value, Property):
-                version[name] = Property(value)
-            else:
-                version[name] = value
-        return version
 
+        # The variable 'first' defines whether we are talking about the
+        # table properties (True) or a about records (False).
+        if first is True:
+            get_datatype = self.get_datatype
+        else:
+            get_datatype = self.get_record_datatype
+
+        # Fix the type
+        to_property = lambda x: x if isinstance(x, Property) else Property(x)
+        for name in properties:
+            value = properties[name]
+            datatype = get_datatype(name)
+
+            # Transform values to properties
+            if is_multilingual(datatype):
+                language = value.parameters['language']
+                version.setdefault(name, [])
+                version[name] = [
+                    x for x in version[name]
+                    if x.parameters['language'] != language ]
+                version[name].append(value)
+            elif datatype.multiple:
+                if type(value) is list:
+                    version[name] = [ to_property(x) for x in value ]
+                else:
+                    version[name] = [to_property(value)]
+            else:
+                version[name] = to_property(value)
+        return version
 
 
     #######################################################################
     # Handlers
     #######################################################################
-    def new(self):
+    def reset(self):
+        self.properties = None
         self.records = []
+        self.added_properties = []
         self.added_records = []
         self.removed_records = []
         # The catalog (for index and search)
         self.catalog = MemoryCatalog()
-        for name, datatype in self.schema.items():
+        for name, datatype in self.record_schema.items():
             index = getattr(datatype, 'index', None)
             if index is not None:
                 field = get_field(index)
                 self.catalog.add_index(name, field)
 
 
+    def new(self):
+        # Add the properties record
+        properties = self.record_class(-1)
+        properties.append({'ts': Property(datetime.now())})
+        self.properties = properties
+
+
     def _load_state_from_file(self, file):
-        self.new()
         # Load the records
         records = self.records
+        properties = self.properties
         n = 0
         version = None
         for name, value, parameters in parse_table(file.read()):
@@ -409,7 +476,13 @@ class Table(File):
                 uid, seq = value.split('/')
                 # Record
                 uid = int(uid)
-                if uid >= n:
+                if uid == -1:
+                    # Tale properties
+                    if properties is None:
+                        properties = self.record_class(uid)
+                        self.properties = properties
+                    record = properties
+                elif uid >= n:
                     # New record
                     records.extend([None] * (uid - n))
                     record = self.record_class(uid)
@@ -421,19 +494,43 @@ class Table(File):
                 # Version
                 if seq == 'DELETED':
                     # Deleted
-                    records[uid] = None
-                    record = None
+                    if uid == -1:
+                        properties = None
+                    else:
+                        records[uid] = None
+                        record = None
                 else:
                     seq = int(seq)
                     if seq > len(record):
                         msg = 'unexpected sequence "%s" for record "%s"'
                         raise ValueError, msg % (seq, uid)
                     record.append(version)
+                # Table or record schema
+                if uid == -1:
+                    get_datatype = self.get_datatype
+                else:
+                    get_datatype = self.get_record_datatype
                 continue
+
+            # Deserialize the parameters
+            for param_name in parameters.keys():
+                param_value = parameters[param_name]
+                param_type = self.get_parameter_datatype(param_name)
+                # Decode
+                param_value = [ param_type.decode(x) for x in param_value ]
+                # Multiple or single
+                if not param_type.multiple:
+                    if len(param_value) > 1:
+                        msg = 'parameter "%s" must be a singleton'
+                        raise ValueError, msg % param_name
+                    param_value = param_value[0]
+                # Update
+                parameters[param_name] = param_value
+
             # Timestamp (ts), Schema, or Something else
-            datatype = self.get_datatype(name)
+            datatype = get_datatype(name)
             value = datatype.decode(value)
-            property = Property(value, parameters)
+            property = Property(value, **parameters)
             if getattr(datatype, 'multiple', False) is True:
                 version.setdefault(name, []).append(property)
             elif name in version:
@@ -450,8 +547,15 @@ class Table(File):
         lines = ['id:%d/%d\n' % (id, seq)]
         names = version.keys()
         names.sort()
+        # Table or record schema
+        if id == -1:
+            get_datatype = self.get_datatype
+        else:
+            get_datatype = self.get_record_datatype
+
+        # Loop
         for name in names:
-            datatype = self.get_datatype(name)
+            datatype = get_datatype(name)
             if getattr(datatype, 'multiple', False) is True:
                 properties = version[name]
             else:
@@ -460,18 +564,27 @@ class Table(File):
                 if property.value is None:
                     continue
                 lines.append(name)
+                # Parameters
                 pnames = property.parameters.keys()
                 pnames.sort()
                 for pname in pnames:
                     pvalues = property.parameters[pname]
-                    pvalues = ','.join(pvalues)
+                    pdatatype = self.get_parameter_datatype(pname)
+                    if pdatatype.multiple:
+                        pvalues = [ pdatatype.encode(x) for x in pvalues ]
+                        pvalues = ','.join(pvalues)
+                    else:
+                        pvalues = pdatatype.encode(pvalues)
                     lines.append(';%s=%s' % (pname, pvalues))
+                # Value
                 value = datatype.encode(property.value)
                 if isinstance(value, Integer):
                     value = str(value)
                 # Escape the value
                 value = escape_data(value)
                 lines.append(':%s\n' % fold_line(value))
+
+        # Ok
         lines.append('\n')
         return ''.join(lines)
 
@@ -479,6 +592,14 @@ class Table(File):
     def to_str(self):
         lines = []
         id = 0
+        # Properties record
+        if self.properties is not None:
+            seq = 0
+            for version in self.properties:
+                version = self._version_to_str(-1, seq, version)
+                lines.append(version)
+                seq += 1
+        # Common record
         for record in self.records:
             if record is not None:
                 seq = 0
@@ -496,8 +617,20 @@ class Table(File):
     # Save (use append for scalability)
     #######################################################################
     def save_state(self):
+        if self.incremental_save is False:
+            File.save_state(self)
+            self.incremental_save = True
+            return
+
+        # Incremental Save
         file = self.safe_open(self.uri, 'a')
         try:
+            # Added properties records
+            for seq in self.added_properties:
+                version = self.properties[seq]
+                version = self._version_to_str(-1, seq, version)
+                file.write(version)
+            self.added_properties = []
             # Added records
             for id, seq in self.added_records:
                 version = self.records[id][seq]
@@ -519,7 +652,7 @@ class Table(File):
 
 
     def save_state_to(self, uri):
-        # TODO: this is a hack, for 0.30 this case should be covered by the
+        # TODO: this is a hack, for 0.50 this case should be covered by the
         # handler's protocol
         File.save_state_to(self, uri)
         if uri == self.uri:
@@ -531,12 +664,12 @@ class Table(File):
     # API / Private
     #######################################################################
     def get_analyser(self, name):
-        datatype = self.get_datatype(name)
+        datatype = self.get_record_datatype(name)
         return get_field(datatype.index)
 
 
     def get_index(self, name):
-        if name not in self.schema:
+        if name not in self.record_schema:
             raise ValueError, 'the field "%s" is not defined' % name
 
         if name not in self.catalog.indexes:
@@ -545,33 +678,24 @@ class Table(File):
         return self.catalog.indexes[name]
 
 
-    def get_field_title(self, name):
-        if name not in self.schema:
-            raise ValueError, 'the field "%s" is not defined' % name
-        if getattr(self, 'form', None) is None:
-            return name
-        for widget in self.form:
-            if widget.name == name:
-                return  getattr(widget, 'title', name)
-
-
     #######################################################################
     # API / Public
     #######################################################################
-    def get_record(self, id, sequence=-1):
-        if id >= len(self.records):
+    def get_record(self, id):
+        try:
+            return self.records[id]
+        except IndexError:
             return None
-        return self.records[id]
 
 
     def add_record(self, kw):
         # Check for duplicate
         for name in kw:
-            datatype = self.get_datatype(name)
+            datatype = self.get_record_datatype(name)
             if getattr(datatype, 'unique', False) is True:
-                if len(self.search(EqQuery(name, kw[name]))) > 0:
-                    title = self.get_field_title(name)
-                    raise ValueError, 'The field %s must be unique' % title
+                search = self.search(PhraseQuery(name, kw[name]))
+                if len(self.search(PhraseQuery(name, kw[name]))) > 0:
+                    raise UniqueError(name, kw[name])
         # Add version to record
         id = len(self.records)
         record = self.record_class(id)
@@ -594,12 +718,11 @@ class Table(File):
             raise LookupError, msg % id
         # Check for duplicate
         for name in kw:
-            datatype = self.get_datatype(name)
+            datatype = self.get_record_datatype(name)
             if getattr(datatype, 'unique', False) is True:
-                search = self.search(EqQuery(name, kw[name]))
+                search = self.search(PhraseQuery(name, kw[name]))
                 if search and (search[0] != self.records[id]):
-                    title = self.get_field_title(name)
-                    raise ValueError, 'The field %s must be unique' % title
+                    raise UniqueError(name, kw[name])
         # Version of record
         version = record[-1].copy()
         version = self.properties_to_dict(kw, version)
@@ -613,6 +736,25 @@ class Table(File):
         self.catalog.index_document(record, id)
 
 
+    def update_properties(self, **kw):
+        record = self.properties
+        if record is None:
+            # if the record doesn't exist
+            # we create it, it's useful during an update
+            record = self.record_class(-1)
+            version = None
+            self.properties = record
+        else:
+            # Version of record
+            version = record[-1].copy()
+        version = self.properties_to_dict(kw, version, first=True)
+        version['ts'] = Property(datetime.now())
+        # Change
+        self.set_changed()
+        self.added_properties.append(len(record))
+        record.append(version)
+
+
     def del_record(self, id):
         record = self.records[id]
         if record is None:
@@ -647,15 +789,74 @@ class Table(File):
             yield self.get_record(id)
 
 
-    def get_value(self, record, name):
+    def get_record_value(self, record, name, language=None):
+        """This is the preferred method for accessing record values.  It
+        returns the value for the given record object and name.
+
+        If the record has not a value with the given name, returns the
+        default value.
+        """
+        # The 'id' is a particular case
+        if name == 'id':
+            return record.id
+
+        # Get the property
+        property = record.get_property(name)
+        datatype = self.get_record_datatype(name)
+
+        # Multilingual properties
+        if is_multilingual(datatype):
+            # Default
+            if property is None:
+                return datatype.get_default()
+            # Language negotiation ('select_language' must be available)
+            if language is None:
+                languages = [ x.parameters['language'] for x in property ]
+                language = select_language(languages)
+            # Miss: default
+            if language is None:
+                # XXX Should send any value?
+                return datatype.get_default()
+            # Hit
+            for x in property:
+                if x.parameters['language'] == language:
+                    return x.value
+            # Default
+            return datatype.get_default()
+
+        # Multiple values
+        if datatype.multiple:
+            # Default
+            if property is None:
+                # FIXME Probably we should check whether the datatype defines
+                # a default value.
+                return []
+            # Hit
+            return [ x.value for x in property ]
+
+        # Simple properties
+        if property is None:
+            return datatype.get_default()
+        return property.value
+
+
+    def get_property(self, name):
+        record = self.properties
+        return record.get_value(name)
+
+
+    def get_property_value(self, name):
         """Return the value if name is in record
         else if name is define in the schema
         return [] is name is a multiple, the default value otherwise.
         """
+        record = self.properties
+        if name == 'id':
+            return record.id
         try:
             return getattr(record, name)
         except AttributeError:
-            if self.schema.has_key(name):
+            if name in self.record_schema:
                 datatype = self.get_datatype(name)
                 if getattr(datatype, 'multiple', False) is True:
                     return []
@@ -687,16 +888,16 @@ class Table(File):
     def update_from_csv(self, data, columns):
         """Update the table by adding record from data
         The input parameters are :
-        
+
         - 'data': the bytes string representation of a CSV.
         - 'columns': the CSV columns used for the mapping between the CSV
           columns and the table schema.
         """
-        schema = self.schema
-        for line in parse(data, columns, schema):
+        record_schema = self.record_schema
+        for line in parse(data, columns, record_schema):
             record = {}
             for index, key in enumerate(columns):
-                if key in schema:
+                if key in record_schema:
                     record[key] = line[index]
             self.add_record(record)
 
