@@ -1,6 +1,7 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2005-2007 Juan David Ibáñez Palomar <jdavid@itaapy.com>
+# Copyright (C) 2006-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
 # Copyright (C) 2007 Hervé Cauwelier <herve@itaapy.com>
+# Copyright (C) 2008 Sylvain Taverne <sylvain@itaapy.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,18 +20,21 @@
 from mimetypes import add_type
 
 # Import from itools
-from domains import DomainAware, register_domain, get_domain
+from domains import register_domain, get_domain, MSG
 from mo import MOFile
-from po import POFile
+from po import POFile, POUnit
 
 
 __all__ = [
-    'DomainAware',
     'register_domain',
     'get_domain',
+    'MSG',
     'MOFile',
-    'POFile']
+    'POFile',
+    'POUnit']
 
 
-add_type('text/x-po', '.po')
-add_type('application/x-mo', '.mo')
+add_type('text/x-gettext-translation', '.po')
+add_type('text/x-gettext-translation-template', '.pot')
+add_type('application/x-gettext-translation', '.mo')
+add_type('application/x-gettext-translation', '.gmo')
