@@ -1,6 +1,8 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2003-2007 Juan David Ibáñez Palomar <jdavid@itaapy.com>
+# Copyright (C) 2003-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
 # Copyright (C) 2007 Hervé Cauwelier <herve@itaapy.com>
+# Copyright (C) 2008 Sylvain Taverne <sylvain@itaapy.com>
+# Copyright (C) 2008 Wynand Winterbach <wynand.winterbach@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +23,9 @@ from time import gmtime, strftime, time
 
 # Import from itools
 from itools.handlers import File, TextFile, register_handler_class
+from itools.srx import TEXT, START_FORMAT, END_FORMAT
+from itools.utils import freeze
+from itools.xml import XMLParser, TEXT as xml_TEXT, START_ELEMENT, END_ELEMENT
 
 
 ###########################################################################
@@ -54,6 +59,7 @@ class POSyntaxError(Exception):
 BLANK = 'blank'
 FUZZY = 'fuzzy'
 COMMENT = 'comment'
+MSGCTXT = 'msgctxt'
 MSGID = 'msgid'
 MSGSTR = 'msgstr'
 STRING = 'string'
@@ -61,6 +67,7 @@ EOF = None
 
 # The regular expressions for the line types
 re_comment = compile('^#.*')
+re_msgctxt = compile(r'^msgctxt *"(.*[^\\]*)"$')
 re_msgid = compile(r'^msgid *"(.*[^\\]*)"$')
 re_msgstr = compile(r'^msgstr *"(.*[^\\]*)"$')
 re_str = compile(r'^"(.*[^\\])"$')
@@ -88,6 +95,12 @@ def get_lines(data):
         # Comment
         elif line.startswith('#'):
             yield COMMENT, line[1:], line_number
+        # msgctxt
+        elif line.startswith('msgctxt'):
+            match = re_msgctxt.match(line)
+            if match is not None:
+                value = match.group(1)
+                yield MSGCTXT, unescape(value), line_number
         # msgid
         elif line.startswith('msgid'):
             match = re_msgid.match(line)
@@ -112,6 +125,77 @@ def get_lines(data):
             raise POSyntaxError(line_number)
 
 
+def encode_source(source):
+    result = []
+    for i, (type, value) in enumerate(source):
+        if type == TEXT:
+            result.append(value)
+        elif type == START_FORMAT:
+            # A lonely tag ?
+            if source[i+1][0] == END_FORMAT:
+                result.append(u"<x id='%d'/>" % value)
+            else:
+                result.append(u"<g id='%d'>" % value)
+        elif type == END_FORMAT:
+            # A lonely tag ?
+            if source[i-1][0] != START_FORMAT:
+                result.append(u'</g>')
+    return u''.join(result)
+
+
+def decode_target(target):
+    # Prepare the regexp
+    re_tags = compile('<(.*?)>')
+    re_g_start = compile("^g id='(.*?)'$")
+    re_g_end = compile("^/g$")
+    re_x = compile("^x id='(.*?)'/$")
+
+    # Parse !
+    result = []
+    id_stack = []
+    tag = True
+    text = ''
+    for block in re_tags.split(target):
+        tag = not tag
+        if tag:
+            # "<g id='...'>"
+            sre = re_g_start.match(block)
+            if sre:
+                id = int(sre.groups()[0])
+                id_stack.append(id)
+                if text:
+                    result.append((TEXT, text))
+                    text = ''
+                result.append((START_FORMAT, id))
+                continue
+            # "</g>"
+            sre = re_g_end.match(block)
+            if sre:
+                if text:
+                    result.append((TEXT, text))
+                    text = ''
+                result.append((END_FORMAT, id_stack.pop()))
+                continue
+            # "<x id='...'>"
+            sre = re_x.match(block)
+            if sre:
+                id = int(sre.groups()[0])
+                if text:
+                    result.append((TEXT, text))
+                    text = ''
+                result.append((START_FORMAT, id))
+                result.append((END_FORMAT, id))
+                continue
+            # Something else
+            text += '<%s>' % block
+            continue
+        text += block
+    # Push the last text
+    if text:
+        result.append((TEXT, text))
+    return result
+
+
 
 ###########################################################################
 # Handler
@@ -128,38 +212,43 @@ def unescape(s):
 
 
 
-class Message(object):
+class POUnit(object):
     """An entry in a PO file has the syntax:
 
     #  translator-comments
     #. automatic-comments
     #: reference...
     #, flag...
+    msgctxt context
     msgid untranslated-string
     msgstr translated-string
 
-    First the comments (optional), then the message id, and finally the
-    message string.
+    First the comments (optional), then the context, the message id, and
+    finally the message string.
 
-    A 'Message' object keeps this information as:
+    A 'POUnit' object keeps this information as:
 
      => references = {<filename>: [<line number>, ...]}
-     => msgid = [<msgid line>, ...]
-     => msgstr = [<msgstr line>, ...]
+     => context = [<msgctxt line>, ...] or None
+     => source = [<msgid line>, ...]
+     => target = [<msgstr line>, ...]
      => comments = [<comment line>, ...]
 
     XXX
     1. Right now the 'comments' list keeps translator comments, automatic
     comments and flags, we should distinguish them.
-    2. Maybe the msgid and msgstr should be kept as a strings instead of lists.
+    2. Maybe the msgid and msgstr should be kept as a strings instead of
+    lists.
     XXX
     """
 
-    def __init__(self, comments, msgid, msgstr, references={}, fuzzy=False):
+    def __init__(self, comments, context, source, target, references={},
+                 fuzzy=False):
         self.comments = comments
         self.references = references
-        self.msgid = msgid
-        self.msgstr = msgstr
+        self.context = context
+        self.source = source
+        self.target = target
         self.fuzzy = fuzzy
 
 
@@ -175,18 +264,36 @@ class Message(object):
         # The Fuzzy flag
         if self.fuzzy:
             s.append('#, fuzzy\n')
+        # The msgctxt
+        if self.context is not None:
+            s.append('msgctxt "%s"\n' % escape(self.context[0].encode(
+                                               encoding)))
+            for string in self.context[1:]:
+                s.append('"%s"\n' % escape(string.encode(encoding)))
         # The msgid
-        s.append('msgid "%s"\n' % escape(self.msgid[0].encode(encoding)))
-        for string in self.msgid[1:]:
+        s.append('msgid "%s"\n' % escape(self.source[0].encode(encoding)))
+        for string in self.source[1:]:
             s.append('"%s"\n' % escape(string.encode(encoding)))
         # The msgstr
-        s.append('msgstr "%s"\n' % escape(self.msgstr[0].encode(encoding)))
-        for string in self.msgstr[1:]:
+        s.append('msgstr "%s"\n' % escape(self.target[0].encode(encoding)))
+        for string in self.target[1:]:
             s.append('"%s"\n' % escape(string.encode(encoding)))
 
         return ''.join(s)
 
 
+    def __repr__(self):
+        msg = "<POUnit object context=%s source=%s target=%s (%s)>"
+        return msg % (self.context, self.source, self.target, self.references)
+
+
+    def __eq__(self, other):
+        return ((other.context == self.context) and
+                (other.source == self.source) and
+                (other.target == self.target))
+
+
+
 skeleton = """# SOME DESCRIPTIVE TITLE.
 # Copyright (C) YEAR THE PACKAGE'S COPYRIGHT HOLDER
 # This file is distributed under the same license as the PACKAGE package.
@@ -209,7 +316,10 @@ msgstr ""
 
 class POFile(TextFile):
 
-    class_mimetypes = ['text/x-po']
+    class_mimetypes = [
+        'text/x-gettext-translation',
+        'text/x-gettext-translation-template',
+        'text/x-po']
     class_extension = 'po'
 
 
@@ -224,7 +334,9 @@ class POFile(TextFile):
     #######################################################################
     def next_entry(self, data):
         # Initialize entry information
-        id, comments, msgid, msgstr, fuzzy = None, [], [], [], False
+        id, comments, context = None, [], None
+        source, target, fuzzy = [], [], False
+
         # Parse entry
         state = 0
         for line_type, value, line_number in get_lines(data):
@@ -241,13 +353,17 @@ class POFile(TextFile):
                 elif line_type == FUZZY and not fuzzy:
                     fuzzy = True
                     state = 1
-                elif line_type == MSGID:
-                    msgid.append(value)
+                elif line_type == MSGCTXT:
+                    context = [value]
                     state = 2
+                elif line_type == MSGID:
+                    source.append(value)
+                    state = 3
                 else:
                     raise POSyntaxError(line_number, line_type)
+
             elif state == 1:
-                # Read comments and wait for the message id
+                # Read comments and wait for the context or message id
                 if line_type == COMMENT:
                     comments.append(value)
                 elif line_type == FUZZY and not fuzzy:
@@ -257,26 +373,41 @@ class POFile(TextFile):
                     fuzzy = False
                     comments = []
                     state = 0
-                elif line_type == MSGID:
-                    msgid.append(value)
+                elif line_type == MSGCTXT:
+                    context = [value]
                     state = 2
-                else:
-                    raise POSyntaxError(line_number, line_type)
-            elif state == 2:
-                # Read the message id and wait for the message string
-                if line_type == STRING:
-                    msgid.append(value)
-                elif line_type == MSGSTR:
-                    msgstr.append(value)
-                    id = ''.join(msgid)
+                elif line_type == MSGID:
+                    source.append(value)
                     state = 3
                 else:
                     raise POSyntaxError(line_number, line_type)
+
+            elif state == 2:
+                # Read the context and wait for the message id
+                if line_type == STRING:
+                    context.append(value)
+                elif line_type == MSGID:
+                    source.append(value)
+                    state = 3
+                else:
+                    raise POSyntaxError(line_number, line_type)
+
             elif state == 3:
+                # Read the message id and wait for the message string
+                if line_type == STRING:
+                    source.append(value)
+                elif line_type == MSGSTR:
+                    target.append(value)
+                    id = ''.join(source)
+                    state = 4
+                else:
+                    raise POSyntaxError(line_number, line_type)
+
+            elif state == 4:
                 # Read the message string
                 if line_type == STRING:
-                    msgstr.append(value)
-                    if id == '':
+                    target.append(value)
+                    if (context, id) == (None, ''):
                         # Parse the header
                         # XXX Right now we only get the encoding, we should
                         # get everything.
@@ -287,26 +418,35 @@ class POFile(TextFile):
                             self.encoding = charset[len('charset='):]
                 elif line_type == BLANK:
                     # End of the entry
-                    yield id, comments, msgid, msgstr, fuzzy, line_number
+                    yield (comments, context, source, target, fuzzy,
+                           line_number)
+                    # Reset
+                    id, comments, context = None, [], None
+                    source, target, fuzzy = [], [], False
                     state = 0
-                    id, comments, msgid, msgstr, fuzzy = None, [], [], [], False
                 elif line_type == COMMENT:
                     # Add entry
-                    self._set_message(msgid, msgstr, comments, {}, fuzzy)
+                    yield (comments, context, source, target, fuzzy,
+                           line_number)
                     # Reset
-                    id, comments, msgid, msgstr, fuzzy = None, [], [], [], False
-                    state = 4
+                    id, comments, context = None, [], None
+                    source, target, fuzzy = [], [], False
+                    state = 5
                 else:
                     raise POSyntaxError(line_number, line_type)
-            elif state == 4:
+
+            elif state == 5:
                 # Discard trailing comments
                 if line_type == COMMENT:
                     pass
                 elif line_type == BLANK:
                     # End of the entry
-                    yield id, comments, msgid, msgstr, fuzzy, line_number
+                    yield (comments, context, source, target, fuzzy,
+                           line_number)
+                    # Reset
+                    id, comments, context = None, [], None
+                    source, target, fuzzy = [], [], False
                     state = 0
-                    id, comments, msgid, msgstr, fuzzy = None, [], [], [], False
                 else:
                     raise POSyntaxError(line_number, line_type)
 
@@ -338,18 +478,28 @@ class POFile(TextFile):
 
         # Add entries
         for entry in self.next_entry(data):
-            entry_id, comments, msgid, msgstr, fuzzy, line_number = entry
+            comments, context, source, target, fuzzy, line_number = entry
+
             # Check for duplicated messages
-            if entry_id in self.messages:
+            if context is not None:
+                first_part = ''.join(context)
+            else:
+                first_part = None
+            second_part = ''.join(source)
+            key = (first_part, second_part)
+            if key in self.messages:
                 raise POError, 'msgid at line %d is duplicated' % line_number
 
             # Get the comments and the msgstr in unicode
             comments = [ unicode(x, self.encoding) for x in comments ]
-            msgid = [ unicode(x, self.encoding) for x in msgid ]
-            msgstr = [ unicode(x, self.encoding) for x in msgstr ]
+
+            if context is not None:
+                context = [ unicode(x, self.encoding) for x in context ]
+            source = [ unicode(x, self.encoding) for x in source ]
+            target = [ unicode(x, self.encoding) for x in target ]
 
             # Add the message
-            self._set_message(msgid, msgstr, comments, {}, fuzzy)
+            self._set_message(context, source, target, comments, None, fuzzy)
 
 
     def to_str(self, encoding='UTF-8'):
@@ -363,65 +513,90 @@ class POFile(TextFile):
     #######################################################################
     # API / Private
     #######################################################################
-    def _set_message(self, msgid, msgstr=[u''], comments=[], references={},
-                     fuzzy=False):
-        if isinstance(msgid, (str, unicode)):
-            msgid = [msgid]
-        if isinstance(msgstr, (str, unicode)):
-            msgstr = [msgstr]
+    def _set_message(self, context, source, target=freeze([u'']),
+                     comments=freeze([]), reference=None, fuzzy=False):
+
+        if context is not None and isinstance(context, (str, unicode)):
+            context = [context]
+        if isinstance(source, (str, unicode)):
+            source = [source]
+        if isinstance(target, (str, unicode)):
+            target = [target]
+
+        # Make the key
+        if context is not None:
+            first_part = ''.join(context)
+        else:
+            first_part = None
+        second_part = ''.join(source)
+        key = (first_part, second_part)
+
+        # Already this entry ?
+        messages = self.messages
+        if key in messages:
+            unit = messages[key]
+        else:
+            unit = POUnit(comments, context, source, target, {}, fuzzy)
+            messages[key] = unit
+
+        # Add the reference and return
+        if reference is None:
+            return unit
+        unit.references.setdefault(reference[0], []).append(reference[1])
+        return unit
 
-        id = ''.join(msgid)
-        self.messages[id] = Message(comments, msgid, msgstr, references, fuzzy)
 
 
     #######################################################################
     # API / Public
     #######################################################################
     def get_msgids(self):
-        """Rerturns all the message ids.
+        """Returns all the (context, msgid).
         """
         return self.messages.keys()
 
 
-    def get_messages(self):
-        """Rerturns all the message (objects of the class <Message>).
+    def get_units(self, srx_handler=None):
+        """Returns all the message (objects of the class <POUnit>).
         """
         return self.messages.values()
 
 
-    def get_msgstr(self, msgid):
-        """Returns the 'msgstr' for the given message id.
+    def get_msgstr(self, source, context=None):
+        """Returns the 'msgstr' for the given (context, msgid).
         """
-        message = self.messages.get(msgid)
+        message = self.messages.get((context, source))
         if message:
-            return ''.join(message.msgstr)
+            return ''.join(message.target)
         return None
 
 
-    def gettext(self, msgid):
+    def gettext(self, source, context=None):
         """Returns the translation of the given message id.
 
-        If the message id is not present in the message catalog, or if it
+        If the context /msgid is not present in the message catalog, or if it
         is marked as "fuzzy", then the message id is returned.
         """
-        message = self.messages.get(msgid)
+
+        message = self.messages.get((context, encode_source(source)))
         if message and not message.fuzzy:
-            msgstr = ''.join(message.msgstr)
-            if msgstr:
-                return msgstr
-        return msgid
+            target = ''.join(message.target)
+            if target:
+                return decode_target(target)
+        return source
 
 
-    def set_message(self, msgid, msgstr=[u''], comments=[], references={},
-                    fuzzy=False):
-        """Adds the given message id to the catalog.
+    def add_unit(self, filename, source, context, line):
+        if not source:
+            return None
 
-        Initialize it with the given, and optional, information: message
-        string, comments, references and fuzzy.
-        """
-        if msgid:
-            self.set_changed()
-            self._set_message(msgid, msgstr, comments, references, fuzzy)
+        # Change
+        self.set_changed()
+
+        source = encode_source(source)
+
+        return self._set_message(context, [source], [u''], [],
+                                 (filename, line))
 
 
 
