@@ -1,8 +1,8 @@
 # -*- coding: UTF-8 -*-
 # Copyright (C) 2006 Hervé Cauwelier <herve@itaapy.com>
 # Copyright (C) 2007 Henry Obein <henry@itaapy.com>
-# Copyright (C) 2007 Juan David Ibáñez Palomar <jdavid@itaapy.com>
 # Copyright (C) 2007 Sylvain Taverne <sylvain@itaapy.com>
+# Copyright (C) 2007-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -40,6 +40,7 @@ class Database(object):
     def __init__(self):
         # The cache
         self.cache = {}
+        self.use_cache = True
         # The state, for transactions
         self.changed = set()
         self.added = set()
@@ -84,14 +85,18 @@ class Database(object):
         return list(names - set(removed) | set(added))
 
 
-    def get_handler(self, reference, cls=None, cache=True):
+    def get_handler(self, reference, cls=None):
         fs, reference = cwd.get_fs_and_reference(reference)
-        to_cache = cache
 
         cache = self.cache
         # Check state
         if reference in self.added:
-            return cache[reference]
+            handler = cache[reference]
+            # cls is good ?
+            if cls is not None and not isinstance(handler, cls):
+                raise LookupError, ('conflict with a handler of type "%s"' %
+                                     handler.__class__)
+            return handler
 
         if reference in self.removed:
             raise LookupError, 'the resource "%s" does not exist' % reference
@@ -123,6 +128,10 @@ class Database(object):
         # Lookup the cache
         if reference in cache:
             handler = cache[reference]
+            # cls is good ?
+            if cls is not None and not isinstance(handler, cls):
+                raise LookupError, ('conflict with a handler of type "%s"' %
+                                     handler.__class__)
             # Not yet loaded or new
             if handler.timestamp is None:
                 return handler
@@ -147,7 +156,7 @@ class Database(object):
         handler.database = self
         handler.uri = reference
         # Update the cache
-        if to_cache is True:
+        if self.use_cache is True:
             cache[reference] = handler
 
         return handler
@@ -165,7 +174,8 @@ class Database(object):
             raise ValueError, 'unexpected folder (only files can be "set")'
 
         if handler.uri is not None:
-            raise ValueError, 'only new files can be added, try to clone first'
+            raise ValueError, ('only new files can be added, '
+                               'try to clone first')
 
         if self.has_handler(reference):
             raise RuntimeError, MSG_URI_IS_BUSY % reference
@@ -214,7 +224,8 @@ class Database(object):
         if isinstance(handler, Folder):
             # Folder
             for name in handler.get_handler_names():
-                self.copy_handler(source.resolve2(name), target.resolve2(name))
+                self.copy_handler(source.resolve2(name),
+                                  target.resolve2(name))
         else:
             # File
             handler = handler.clone()
@@ -240,7 +251,8 @@ class Database(object):
         if isinstance(handler, Folder):
             # Folder
             for name in handler.get_handler_names():
-                self.move_handler(source.resolve2(name), target.resolve2(name))
+                self.move_handler(source.resolve2(name),
+                                  target.resolve2(name))
         else:
             # Load if needed
             if handler.timestamp is None and handler.dirty is None:
@@ -262,6 +274,18 @@ class Database(object):
                 self.removed.add(source)
 
 
+    #######################################################################
+    # API / Cache
+    #######################################################################
+    def set_use_cache(self, cache):
+        self.use_cache = bool(cache)
+
+
+    def add_to_cache(self, uri, handler):
+        if self.use_cache is True:
+            self.cache[uri] = handler
+
+
     #######################################################################
     # API / Safe VFS operations (not really safe)
     #######################################################################
