@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright (C) 2005 Nicolas Oyez <nicoyez@gmail.com>
-# Copyright (C) 2005-2007 Juan David Ibáñez Palomar <jdavid@itaapy.com>
+# Copyright (C) 2005-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,16 +20,16 @@ import unittest
 from unittest import TestCase
 
 # Import from itools
-from itools.xliff import XLIFF
+from itools.xliff import XLFFile
 
 
 
-class TMXTestCase(TestCase):
+class XLIFFTestCase(TestCase):
 
     def test_input(self):
         """Test input.
         """
-        xliff = XLIFF('tests/gettext_en_es.xlf')
+        xliff = XLFFile('tests/gettext_en_es.xlf')
         xliff.to_str()
 
 
